/*******************************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   Matthew Webster           initial implementation      
 *******************************************************************************/

package demo.eclipse.extension.point;

import hello.ITest;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.Platform;


public aspect Test {

	before () : execution(public static void main(String[])) {
		System.err.println("? Test.before()");
		for (Iterator i = tests.iterator(); i.hasNext();) {
			ITest test = (ITest)i.next();
			test.test(thisJoinPoint.toString());
		}
	}

	public static final String TEST_EXTENSION = "demo.hello.tests"; //$NON-NLS-1$
	
	private static List /* ITest */tests;
	
	private static void initialiseTests() {
		System.err.println("> Test.initialiseTests()");
		
		tests = new ArrayList();
		IExtensionPoint exP = Platform.getExtensionRegistry()
				.getExtensionPoint(TEST_EXTENSION);
		IExtension[] exs = exP.getExtensions();

		for (int i = 0; i < exs.length; i++) {
			IConfigurationElement[] ces = exs[i].getConfigurationElements();
			for (int j = 0; j < ces.length; j++) {
				try {
					Object ext = ces[j].createExecutableExtension("class"); //$NON-NLS-1$
					if (ext instanceof ITest) {
						String name = ces[j].getAttribute("name"); //$NON-NLS-1$
						String id = ces[j].getAttribute("id"); //$NON-NLS-1$
						System.err.println("- Test.initialiseTests() ext=" + ext);
						tests.add(ext);
					}
				} catch (CoreException ex) {
					ex.printStackTrace();
				}
			}
		}
		
		System.err.println("< Test.initialiseTests() " + tests.size());
	}

	static {
		initialiseTests();
	}
}
