/*******************************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   Matthew Webster           initial implementation      
 *******************************************************************************/

package demo.eclipse.tracing;

import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceEvent;
import org.osgi.framework.ServiceListener;
import org.osgi.framework.ServiceReference;
import org.osgi.service.packageadmin.PackageAdmin;

public class EclipseTraceBundle implements BundleActivator {

	private BundleContext bundleContext;
	
	public void start(BundleContext context) throws Exception {
		initialize(context);
	}

	public void stop(BundleContext context) throws Exception {
		// TODO Auto-generated method stub

	}
	
	private void initialize (BundleContext context) {
		this.bundleContext = context;

		initializePackageAdminService();
		
		String  traceFilter = "(objectclass="+ITrace.class.getName()+")"; 

		ServiceListener traceListener = new ServiceListener() {

			public void serviceChanged(ServiceEvent event) {
//				System.err.println("- ServiceListener.serviceChanged() event=" + event);
				if(event.getType() == ServiceEvent.REGISTERED) {
					initializeTraceService();
				}
				else if(event.getType() == ServiceEvent.UNREGISTERING) {
					finalizeTraceService();
				}
			}
			
		};

		try {
			bundleContext.addServiceListener(traceListener,traceFilter);
		}
		catch (Exception ex) {
			ex.printStackTrace();
		}
	}
	
	protected void initializeTraceService () {
//		System.err.println("> EclipseTraceBundle.initializeTraceService()");

		try {
			ServiceReference ref = bundleContext.getServiceReference(ITrace.class.getName());
//			System.err.println("- EclipseTraceBundle.initializeTraceService() ref=" + ref);
			if (ref != null) {
				ITrace traceService = (ITrace)bundleContext.getService(ref);
//				System.err.println("- EclipseTraceBundle.initializeTraceService() traceService=" + traceService);
				EclipseTracing.aspectOf().setTraceService(traceService);
			}
		}
		catch (Exception ex) {
			ex.printStackTrace();
		}
		
//		System.err.println("< EclipseTraceBundle.initializeTraceService()");
	}
	
	protected void finalizeTraceService () {
//		System.err.println("> EclipseTraceBundle.finalizeTraceService()");

		EclipseTracing.aspectOf().setTraceService(null);
		
//		System.err.println("< EclipseTraceBundle.finalizeTraceService()");
	}
	
	private void initializePackageAdminService () {
		ServiceReference ref = bundleContext.getServiceReference(PackageAdmin.class.getName());
		System.err.println("? EclipseTraceBundle.initializePackageAdminService() ref=" + ref);
		if (ref != null) {
			PackageAdmin service = (PackageAdmin)bundleContext.getService(ref);
			System.err.println("? EclipseTraceBundle.initializePackageAdminService() service=" + service);
			EclipseTracing.aspectOf().setPackageAdminService(service);
		}
	}

}
