/*******************************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   Matthew Webster           initial implementation      
 *******************************************************************************/

package demo.eclipse.tracing;

import org.aspectj.lib.tracing.SimpleTracing;
import org.osgi.service.packageadmin.PackageAdmin;

public aspect EclipseTracing extends SimpleTracing {
	
	public final static pointcut bundleActivation () :
		execution(public * org.osgi.framework.BundleActivator.*(..));
	
	protected pointcut tracingScope() :
		bundleActivation() && !within(demo.eclipse.tracing..*);

	private ITrace traceService = null;
	private PackageAdmin packageAdmin;

	public EclipseTracing () {
		System.err.println("? EclipseTracing.EclipseTracing()");
	}
	
	public void setTraceService (ITrace trace) {
		System.err.println("? EclipseTracing.setTraceService() trace=" + trace);
		this.traceService = trace;
	}
	
	public void setPackageAdminService (PackageAdmin packageAdmin) {
		System.err.println("? EclipseTracing.setPackageAdminService() packageAdmin=" + packageAdmin);
		this.packageAdmin = packageAdmin;
	}

	private String getBundleSymbolicName (Class clazz) {
//		ClassLoader loader = clazz.getClassLoader();
//		BaseClassLoader baseClassLoader = (BaseClassLoader)loader;
//		Bundle bundle = baseClassLoader.getClasspathManager().getBaseData().getBundle();
//		String bundleName = bundle.getSymbolicName();

		String bundleName = "demo.eclipse.tracing";
		if (packageAdmin != null) {
			bundleName = packageAdmin.getBundle(clazz).getSymbolicName();
		}
		
		return bundleName;
	}
	
	public void println (Class clazz, String s) {
//		System.err.println(s);
		if (traceService != null) {
			traceService.println("[" + getBundleSymbolicName(clazz) + "] " + s);
//			traceService.println(clazz,s);
		}
	}
	
}
