/*******************************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   Matthew Webster           initial implementation      
 *******************************************************************************/

package demo.eclipse.translators;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseTrackListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;
import org.eclipse.swt.widgets.Widget;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowPulldownDelegate;

import demo.eclipse.swt.translators.*;

public class LanguageSelectionMenu implements IWorkbenchWindowPulldownDelegate,
		SelectionListener, MouseTrackListener {

	public static boolean verbose = false;
	
	public static final String TRANSLATORS_EXTENSION = "demo.eclipse.translators.extension.point.translators"; //$NON-NLS-1$
	
	private Map translators = new HashMap();
	private ITranslationProvider activeTranslator;
	private String activeLanguage;
	
	private Widget widget;
	private IToolTipText[] widgets;
	private String[] toolTips;
	
	public LanguageSelectionMenu () {
		if (verbose) System.err.println("> LanguageSelectionMenu.LanguageSelectionMenu()");
		
		initialize();
//		IExtensionPoint[] extensionPoints = Platform.getExtensionRegistry().getExtensionPoints();
//		for (int i = 0; i < extensionPoints.length; i++) {
//			if (verbose) System.err.println("- LanguageSelectionMenu.LanguageSelectionMenu() " + extensionPoints[i].getUniqueIdentifier());
//		}
		
		TooltipInterceptor.addMouseTrackListener(this);

		if (verbose) System.err.println("< LanguageSelectionMenu.LanguageSelectionMenu()");
	}
	
	private void initialize () {
		if (verbose) System.err.println("> LanguageSelectionMenu.initialize()");

		try {
			IExtensionPoint exP = Platform.getExtensionRegistry().getExtensionPoint(TRANSLATORS_EXTENSION);
			IExtension[] exs = exP.getExtensions();

			for (int i = 0; i < exs.length; i++) {
				IConfigurationElement[] ces = exs[i].getConfigurationElements();
				for (int j = 0; j < ces.length; j++) {
					try {
						Object ext = ces[j].createExecutableExtension("class"); //$NON-NLS-1$
						if (ext instanceof ITranslationProvider	) {
//							String name = ces[j].getAttribute("name"); //$NON-NLS-1$
//							String id = ces[j].getAttribute("id"); //$NON-NLS-1$
							String language = ces[j].getAttribute("language"); //$NON-NLS-1$
							if (verbose) System.err.println("- LanguageSelectionMenu.initialize() ext=" + ext);
							if (!translators.containsKey(language)) {
								translators.put(language,ext);
								if (activeLanguage == null) activeLanguage = language;
							}
						}
					} catch (CoreException ex) {
						ex.printStackTrace();
					}
				}
			}

			if (activeLanguage != null) activeTranslator = (ITranslationProvider)translators.get(activeLanguage);
		}
		catch (Exception ex) {
			ex.printStackTrace();
		}

		if (verbose) System.err.println("< LanguageSelectionMenu.initialize()() " + translators.size());
	}
	
	public Menu getMenu(Control parent) {
		if (verbose) System.err.println("> LanguageSelectionMenu.getMenu()");
		Menu menu = new Menu(parent);
		
		String[] languages = getLanguages();
		String activeLanguage = getActiveLanguage();
		for (int i = 0; i < languages.length; i++) {
			String language = languages[i];
			MenuItem m = new MenuItem(menu,SWT.CHECK);
			m.setText(language);
			m.setData(language);
			if (language.equals(activeLanguage)) m.setSelection(true);
			m.addSelectionListener(this);
		}

		if (verbose) System.err.println("< LanguageSelectionMenu.getMenu() " + menu);
		return menu;
	}

	public void dispose() {
		// TODO Auto-generated method stub

	}

	public void init(IWorkbenchWindow window) {
		// TODO Auto-generated method stub

	}

	public void run(IAction action) {
		// TODO Auto-generated method stub

	}

	public void selectionChanged(IAction action, ISelection selection) {
	}

	public void widgetDefaultSelected(SelectionEvent e) {
	}

	public void widgetSelected(SelectionEvent e) {
		Object src = e.getSource();
		if (src instanceof MenuItem) {
			if (verbose) System.err.println("- LanguageSelectionMenu.widgetSelected() e=" + e);
			MenuItem w = (MenuItem) src;
			String language = (String)w.getData();
//			if (w.getSelection()) language = "";
			setActiveLanguage(language);
		}
	}
	
	public String[] getLanguages () {
		Set keySet = translators.keySet();
		String[] languages = new String[keySet.size()];
		keySet.toArray(languages);
		return languages;
	}
	
	public String getActiveLanguage () {
		return activeLanguage;
	}
	
	public void setActiveLanguage (String language) {
		setActiveTranslator(language);
	}
	
	private void setActiveTranslator (String language) {
		if (activeLanguage != language) {
			activeLanguage = language;
			activeTranslator = (ITranslationProvider)translators.get(activeLanguage);
		}
		else {
			activeLanguage = "";
			activeTranslator = null;
		}
	}
	
	public void mouseEnter(MouseEvent e) {
		if (verbose) System.err.println("> TooltipInterceptor.mouseEnter() e=" + e + ", widget=" + e.widget);

		if (activeTranslator != null) try {
			if (widget != null) restore();

			if (e.widget instanceof ToolBar) {
				ToolBar toolBar = (ToolBar)e.widget;
				widget = toolBar;
				ToolItem[] toolItems = toolBar.getItems();
				if (verbose) System.err.println("- TooltipInterceptor.mouseEnter() items=" + Arrays.asList(toolItems));
				int length = toolItems.length;
				widgets = new IToolTipText[length];
				toolTips = new String[length];
				for (int i = 0; i < toolItems.length; i++) {
					ToolItem widget = toolItems[i];
					widgets[i] = (IToolTipText)widget;
					String text = widget.getToolTipText();
					toolTips[i] = text;
//					toolItems[i].setToolTipText(lookup(text) + " (" + text + ")");
					if (text != null) toolItems[i].setToolTipText(activeTranslator.translate(text) + " (" + text + ")");
				}
			}
			else if (e.widget instanceof Control) {
				Control control = (Control)e.widget;
				widget = control;
				if (verbose) System.err.println("- TooltipInterceptor.mouseEnter() control=" + control);
				widgets = new IToolTipText[1];
				toolTips = new String[1];
				widgets[0] = (IToolTipText)control;
				String text = control.getToolTipText();
				toolTips[0] = text;
				if (text != null) control.setToolTipText(activeTranslator.translate(text) + " (" + text + ")");
			}
		}
		catch (Exception ex) {
			ex.printStackTrace();
		}

		if (verbose) System.err.println("< TooltipInterceptor.mouseEnter() e=" + e + ", widget=" + e.widget);
	}
	
	private void restore () {
		if (verbose) System.err.println("- TooltipInterceptor.restore()");
		for (int i = 0; i < widgets.length; i++) {
			widgets[i].setToolTipText(toolTips[i]);
		}
		
		widget = null;
	}

	public void mouseExit(MouseEvent e) {
		if (verbose) System.err.println("> TooltipInterceptor.mouseExit() e=" + e);

		if (e.widget.equals(widget)) try {
			if (verbose) System.err.println("- TooltipInterceptor.mouseExit() widget=" + widget);
			restore();
		}
		catch (Exception ex) {
			ex.printStackTrace();
		}
		
		if (verbose) System.err.println("< TooltipInterceptor.mouseExit() e=" + e);
	}

	public void mouseHover(MouseEvent e) {
//		System.err.println("? TooltipInterceptor.mouseHover() e=" + e);
	}

//	public void mouseEnter(MouseEvent e) {
//		System.out.println("LanguageSelectionMenu.mouseEnter()");
//	}
//
//	public void mouseExit(MouseEvent e) {
//		System.out.println("LanguageSelectionMenu.mouseExit()");
//	}
//
//	public void mouseHover(MouseEvent e) {
//		// TODO Auto-generated method stub
//		System.out.println("LanguageSelectionMenu.mouseHover()");
//	}

}
