/*******************************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   Matthew Webster           initial implementation      
 *******************************************************************************/

package demo.eclipse.ui.workbench;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProjectNature;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.ui.IEditorDescriptor;
import org.eclipse.ui.activities.WorkbenchActivityHelper;
import org.eclipse.ui.internal.util.Util;

public aspect EditorDescriptorWithNature {
	
	private final static boolean verbose = false;

	/*
	 * 1. Define a new editor attribute
	 */
	public static String ATT_NATURE = "nature"; //$NON-NLS-1$

	/*
	 * 2. Define a new ITD field to hold the nature
	 */
	private String IEditorDescriptor.nature;
	
	/*
	 * 3. Populate new nature field from editor configuration
	 */
	before (IEditorDescriptor editor, IConfigurationElement element) : execution(void IEditorDescriptor+.setConfigurationElement(..)) && this(editor) && args(element) {
		if(verbose) System.err.println("> EditorDescriptorWithNature.beforeSetConfigurationElement() editor=" + editor.getId());
  
		editor.nature = Util.safeString(element.getAttribute(ATT_NATURE));
		
		if(verbose) System.err.println("< EditorDescriptorWithNature.beforeSetConfigurationElement() nature=" + editor.nature);
	}

	/*
	 * 4. Intercept opening of editor at the point when we know the IFile 
	 */
	private pointcut getDefaultEditorFromIDE (IFile file) :
		within(org.eclipse.ui.ide.*) && execution(public IEditorDescriptor getDefaultEditor(IFile, boolean)) && args(file,boolean);

	/*
	 * 5. Intercept contribution filtering  
	 */
	private pointcut filterItem (Object object) :
		execution(public boolean WorkbenchActivityHelper.filterItem(..)) && args(object);
	
	/*
	 * 6. Use "wormhole" pattern to determine IProjectNature and new nature
	 * attribute to dynamically filter editors  
	 */
	boolean around (Object object, IFile file) : filterItem (object) 
		&& cflow(getDefaultEditorFromIDE (file)) {
		if(verbose) System.err.println("> EditorDescriptorWithNature.aroundFilterItem() editor=" + object + ", file=" + file);
		
		boolean filtered = proceed(object,file);
		if (!filtered && object instanceof IEditorDescriptor) try {
			IEditorDescriptor editor = (IEditorDescriptor)object;
			String nature = editor.nature;
			IProjectNature projectNature = file.getProject().getNature(nature);
			if(verbose) System.err.println("? EditorDescriptorWithNature.aroundFilterItem() nature=" + nature + ", projectNature=" + projectNature);
			if (nature != null && nature.length() != 0 && projectNature == null) {
				filtered = true;
			}
		}
		catch (CoreException ex) {
			ex.printStackTrace();
		}
		
		if(verbose) System.err.println("< EditorDescriptorWithNature.aroundFilterItem() " + filtered);
		return filtered;
	}

	/*
	 * Debugging
	 */
	before (IFile file) : getDefaultEditorFromIDE (file)  {
		if(verbose) System.err.println("? EditorDescriptorWithNature.beforeGetDefaultEditor() file=" + file);
	}
	
	before (String editorId) : execution(public * openEditor(String, ..)) && args(editorId, ..) {
		if(verbose) System.err.println("? EditorDescriptorWithNature.openEditor() editorId=" + editorId);
	}
	
//	private pointcut getDeclaredDefaultEditorsFromFileEditorMapping () :
//		execution(public IEditorDescriptor[] getDeclaredDefaultEditors());
//	
//	IEditorDescriptor[] around (Object obj) : getDeclaredDefaultEditorsFromFileEditorMapping () 
//		&& cflow(getDefaultEditorFromIDE (obj)) {
//	System.err.println("> EditorDescriptorWithNature.aroundGetDeclaredDefaultEditors() " + this + " file=" + obj);
//
//		IEditorDescriptor[] editors = proceed(obj);
//		
//		System.err.println("< EditorDescriptorWithNature.aroundGetDeclaredDefaultEditors() " + this + " editors=" + Arrays.asList(editors));
//		return editors;
//	}
}
